__title__ = 'Django REST Framework - Contact'
__version__ = '0.0.2'
__author__ = 'Civil Machines Technologies Private Limited'
__license__ = 'GPLv3'

default_app_config = 'drf_contact.apps.DRFContactConfig'



from django.contrib import admin
from .models import ContactDetail


class MyModelAdmin(admin.ModelAdmin):

    def get_readonly_fields(self, request, obj=None):
        """
        Makes created_by & create_date readonly when editing.
        :param request: HttpResponse
        :param obj: Model's Object
        :return: tuple
        """
        if not obj:
            return ()
        return 'created_by', 'create_date'


admin.site.register(ContactDetail, MyModelAdmin)



from django.conf.urls import url
from . import views


app_name = "drf_contact"


urlpatterns = [
    # ex: api/contacts/show/
    url(r'^show/', views.ShowContacts.as_view(), name='show-contact'),
    # ex: api/contacts/add/
    url(r'^add/', views.AddContacts.as_view(), name='add-contact'),
]



from drfaddons.models import CreateUpdateModel


class ContactDetail(CreateUpdateModel):
    """
    A custom ContactDetails model that keeps a record of all the details of a contact.
    """
    from django.utils.text import gettext_lazy as _
    from django.db import models

    name = models.CharField(_('Unique UserName'), max_length=254)
    email = models.EmailField(_('Email Address'), null=True)
    mobile = models.CharField(_('Mobile Number'), max_length=15, null=True)

    @property
    def user(self):
        from django.contrib.auth import get_user_model

        usr = get_user_model()

        try:
            email_user = usr.objects.get(email=self.email)
        except usr.DoesNotExist:
            email_user = None

        try:
            mobile_user = usr.objects.get(mobile=self.mobile)
        except usr.DoesNotExist:
            mobile_user = None

        # TODO: Handle following situation in better way.
        if not email_user and not mobile_user:
            return email_user or mobile_user
        elif email_user and not mobile_user:
            # TODO: Why mobile of contact and user is not same?
            return email_user or mobile_user
        elif not email_user and mobile_user:
            # TODO: Why mobile of contact and user is not same?
            return email_user or mobile_user
        else:
            if email_user == mobile_user:
                # This is ideal case
                return email_user or mobile_user
            else:
                # TODO: How two users can exist where in the contact is same?
                return email_user

    def __str__(self):
        """
        Returns string representation of the name.
        """
        return self.name

    class Meta:
        from django.utils.text import gettext_lazy as _

        unique_together = ('created_by', 'name')
        verbose_name = _('Contact Detail')
        verbose_name_plural = _('Contact Details')



from rest_framework import serializers


class ShowContactDetailSerializer(serializers.ModelSerializer):
    """
    ShowContactDetailSerializer is a model serializer that includes the details of a contact.
    Returns
    -------
        returns a dictionary containing::
            'id' : int
            'name' : str
            'email' : str
            'mobile' : str
    """

    class Meta:
        from .models import ContactDetail

        model = ContactDetail
        fields = ('id', 'name', 'email', 'mobile')


class AddContactDetailSerializer(serializers.ModelSerializer):
    """
    AddContactDetailSerializer is a model serializer that includes the fields required for adding a new contact.
    Returns
    -------
        returns a dictionary containing::
            'name' : str
            'email' : str
            'mobile' : str
    """

    class Meta:
        from .models import ContactDetail

        model = ContactDetail
        fields = ('name', 'email', 'mobile')



# Generated by Django 2.0.7 on 2018-07-06 12:56

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import drfaddons.datatypes


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactDetail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('update_date', drfaddons.datatypes.UnixTimestampField(auto_created=True)),
                ('create_date', drfaddons.datatypes.UnixTimestampField(auto_now_add=True)),
                ('name', models.CharField(max_length=254, verbose_name='Unique UserName')),
                ('email', models.EmailField(max_length=254, null=True, verbose_name='Email Address')),
                ('mobile', models.CharField(max_length=15, null=True, verbose_name='Mobile Number')),
                ('created_by', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Contact Detail',
                'verbose_name_plural': 'Contact Details',
            },
        ),
        migrations.AlterUniqueTogether(
            name='contactdetail',
            unique_together={('created_by', 'name')},
        ),
    ]



from rest_framework.generics import ListAPIView, CreateAPIView


class ShowContacts(ListAPIView):
    """
    This view is to show the details of a contact.
    """
    from .serializers import ShowContactDetailSerializer
    from rest_framework.filters import SearchFilter
    from .models import ContactDetail
    from drfaddons.filters import IsOwnerFilterBackend
    from django_filters.rest_framework import DjangoFilterBackend
    from rest_framework.pagination import PageNumberPagination
    from rest_framework.permissions import IsAuthenticated
    from rest_framework.renderers import JSONRenderer
    from rest_framework.parsers import JSONParser

    queryset = ContactDetail.objects.all().order_by('-create_date')
    serializer_class = ShowContactDetailSerializer
    filter_backends = (IsOwnerFilterBackend, DjangoFilterBackend, SearchFilter)
    search_fields = ('^name',)
    pagination_class = PageNumberPagination
    page_size = 10
    permission_classes = (IsAuthenticated, )
    renderer_classes = (JSONRenderer, )
    parser_classes = (JSONParser, )


class AddContacts(CreateAPIView):
    """
    This view is to add new contacts.
    """
    from .serializers import AddContactDetailSerializer
    from rest_framework.permissions import IsAuthenticated
    from rest_framework.renderers import JSONRenderer
    from rest_framework.parsers import JSONParser

    serializer_class = AddContactDetailSerializer
    permission_classes = (IsAuthenticated, )
    renderer_classes = (JSONRenderer, )
    parser_classes = (JSONParser, )

    def perform_create(self, serializer):
        serializer.save(created_by=self.request.user)



from setuptools import find_packages, setup

with open('README.md', 'r') as readme:
    README = readme.read()

setup(
    name='drf_contact',
    version=__import__('drf_contact').__version__,
    author=__import__('drf_contact').__author__,
    author_email='pypidev@civilmachines.com',
    description='Contact APP for Django REST Framework with API Views.',
    long_description=README,
    long_description_content_type="text/markdown",
    license=__import__('drf_contact').__license__,
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/civilmachines/drf-contact',
    install_requires=open('requirements.txt').read().split(),
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP'
    ],
)
